@@ -94,6 +94,7 @@ static int alarmdebounce = 500;
 static int vpmsupport = 1;
 static int timer_1_ms = 2000;
 static int timer_3_ms = 30000;
+static char *companding = "alaw";
 
 #if !defined(mmiowb)
 #define mmiowb() barrier()
@@ -142,8 +143,6 @@ static struct devtype hfc4s_EV = {"CCD HFC-4S Eval. Board", .ports = 4,
  
 #define CARD_HAS_EC(card) ((card)->card_type == B410P)
 
-static int echocan_create(struct dahdi_chan *chan, struct dahdi_echocanparams *ecp,
-			   struct dahdi_echocanparam *p, struct dahdi_echocan_state **ec);
 static void echocan_free(struct dahdi_chan *chan, struct dahdi_echocan_state *ec);
 
 static const struct dahdi_echocan_features my_ec_features = {
@@ -158,7 +157,7 @@ static const struct dahdi_echocan_ops my_ec_ops = {
 };
 
 #if 0
-static const char *wcb4xxp_rcsdata = "$RCSfile: base.c,v $ $Revision: 8423 $";
+static const char *wcb4xxp_rcsdata = "$RCSfile: base.c,v $ $Revision: 9702 $";
 static const char *build_stamp = "" __DATE__ " " __TIME__ "";
 #endif
 
@@ -685,12 +684,18 @@ static void ec_init(struct b4xxp *b4)
 			ec_write(b4, i, 0x33 - j, (mask >> (j << 3)) & 0xff);
 
 /* Setup convergence rate */
-		if (DBG)
-			dev_info(b4->dev, "setting A-law mode\n");
-
 		b = ec_read(b4, i, 0x20);
 		b &= 0xe0;
-		b |= 0x13;
+		b |= 0x12;
+		if (!strcasecmp(companding, "alaw")) {
+			if (DBG)
+				dev_info(b4->dev, "Setting alaw mode\n");
+			b |= 0x01;
+		} else {
+			if (DBG)
+				dev_info(b4->dev, "Setting ulaw mode");
+		}
+
 		ec_write(b4, i, 0x20, b);
 		if (DBG)
 			dev_info(b4->dev, "reg 0x20 is 0x%02x\n", b);
@@ -1119,12 +1124,17 @@ static void b4xxp_set_sync_src(struct b4xxp *b4, int port)
 {
 	int b;
 
+#if 0
+	printk("Setting sync to be port %d\n", (port >= 0) ? port + 1 : port);
+#endif
+
 	if (port == -1) 		/* automatic */
 		b = 0;
 	else
 		b = (port & V_SYNC_SEL_MASK) | V_MAN_SYNC;
 
 	b4xxp_setreg8(b4, R_ST_SYNC, b);
+	b4->syncspan = port;
 }
 
 /*
@@ -1139,16 +1149,58 @@ static int b4xxp_find_sync(struct b4xxp *b4)
 	src = -1;		/* default to automatic */
 
 	for (i=0; i < b4->numspans; i++) {
+		if (DBG)
+			dev_info(b4->dev, "Checking sync pos %d, have span %d\n", i, b4->spans[i].sync);
 		psrc = b4->spans[i].sync;
 		if (psrc > 0 && !b4->spans[psrc - 1].span.alarms) {
+			if (DBG)
+				dev_info(b4->dev, "chosen\n");
 			src = psrc;
 			break;
 		}
 	}
 
-	return src - 1;
+	if (src >= 0)
+		return src - 1;
+	else
+		return src;
 }
 
+#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18))
+static ssize_t b4_timing_master_show(struct device *dev,
+				     struct device_attribute *attr,
+				     char *buf)
+{
+	struct b4xxp *b4 = dev_get_drvdata(dev);
+	return sprintf(buf, "%d\n", b4->syncspan);
+}
+
+static DEVICE_ATTR(timing_master, 0400, b4_timing_master_show, NULL);
+
+static void create_sysfs_files(struct b4xxp *b4)
+{
+	int ret;
+	ret = device_create_file(b4->dev,
+				 &dev_attr_timing_master);
+	if (ret) {
+		dev_info(b4->dev,
+			"Failed to create device attributes.\n");
+	}
+}
+
+static void remove_sysfs_files(struct b4xxp *b4)
+{
+	device_remove_file(b4->dev,
+			   &dev_attr_timing_master);
+}
+
+#else
+
+static inline void create_sysfs_files(struct b4xxp *b4) { return; }
+static inline void remove_sysfs_files(struct b4xxp *b4) { return; }
+
+#endif /* LINUX_KERNEL > 2.6.18 */
+
 /*
  * allocates memory and pretty-prints a given S/T state engine state to it.
  * calling routine is responsible for freeing the pointer returned!
@@ -1274,13 +1326,16 @@ static void hfc_update_st_timers(struct b4xxp *b4)
 		}
 
 		if (s->newalarm != s->span.alarms && time_after_eq(b4->ticks, s->alarmtimer)) {
-			if (!s->te_mode || !teignorered) {
-				s->span.alarms = s->newalarm;
+			s->span.alarms = s->newalarm;
+			if ((!s->newalarm && teignorered) || (!teignorered)) {
 				dahdi_alarm_notify(&s->span);
-				if (DBG_ALARM)
-					dev_info(b4->dev, "span %d: alarm %d debounced\n", i + 1, s->newalarm);
-				if (!s->te_mode)
-					b4xxp_set_sync_src(b4, b4xxp_find_sync(b4));
+			}
+			b4xxp_set_sync_src(b4, b4xxp_find_sync(b4));
+			if (DBG_ALARM) {
+				dev_info(b4->dev,
+					 "span %d: alarm %d "
+					 "debounced\n",
+					 i + 1, s->newalarm);
 			}
 		}
 	}
@@ -1375,7 +1430,7 @@ static void hfc_handle_state(struct b4xxp_span *s)
 /* read in R_BERT_STA to determine where our current sync source is */
 	newsync = b4xxp_getreg8(b4, R_BERT_STA) & 0x07;
 	if (newsync != b4->syncspan) {
-		if (printk_ratelimit())
+		if (printk_ratelimit() || DBG)
 			dev_info(b4->dev, "new card sync source: port %d\n", newsync + 1);
 		b4->syncspan = newsync;
 	}
@@ -1784,8 +1839,10 @@ static int hdlc_tx_frame(struct b4xxp_span *bspan)
 		for (i=0; i < size; i++)
 			printk("%02x%c", buf[i], (i < (size - 1)) ? ' ' : '\n');
 
-		if (size && res != 0)
-			pr_info("Transmitted frame %d on span %d\n", bspan->frames_out - 1, bspan->port);
+		if (size && res != 0) {
+			pr_info("Transmitted frame %d on span %d\n",
+				bspan->frames_out - 1, bspan->port + 1);
+		}
 	}
 
 	return(res == 0);
@@ -1850,6 +1907,8 @@ static void b4xxp_init_stage1(struct b4xxp *b4)
 	flush_pci();
 
 	udelay(100);				/* wait a bit for clock to settle */
+
+	create_sysfs_files(b4);
 }
 
 /*
@@ -1875,6 +1934,9 @@ static void b4xxp_init_stage2(struct b4xxp *b4)
 	flush_pci();
 	b4xxp_setreg8(b4, R_PCM_MD1, V_PLL_ADJ_00 | V_PCM_DR_2048);
 
+	b4xxp_setreg8(b4, R_PWM_MD, 0xa0);
+	b4xxp_setreg8(b4, R_PWM0, 0x1b);
+
 /*
  * set up the flow controller.
  * B channel map: (4 ports cards with Hardware Echo Cancel present & active)
@@ -2078,12 +2140,17 @@ static void b4xxp_update_leds(struct b4xxp *b4)
 	}
 }
 
-static int echocan_create(struct dahdi_chan *chan, struct dahdi_echocanparams *ecp,
-			  struct dahdi_echocanparam *p, struct dahdi_echocan_state **ec)
+static int b4xxp_echocan_create(struct dahdi_chan *chan,
+				struct dahdi_echocanparams *ecp,
+				struct dahdi_echocanparam *p,
+				struct dahdi_echocan_state **ec)
 {
-	struct b4xxp_span *bspan = chan->span->pvt;
+	struct b4xxp_span *bspan = container_of(chan->span, struct b4xxp_span, span);
 	int channel;
 
+	if (!vpmsupport || !CARD_HAS_EC(bspan->parent))
+		return -ENODEV;
+
 	if (chan->chanpos == 3) {
 		printk(KERN_WARNING "Cannot enable echo canceller on D channel of span %d; failing request\n", chan->span->offset);
 		return -EINVAL;
@@ -2110,7 +2177,7 @@ static int echocan_create(struct dahdi_chan *chan, struct dahdi_echocanparams *e
 
 static void echocan_free(struct dahdi_chan *chan, struct dahdi_echocan_state *ec)
 {
-	struct b4xxp_span *bspan = chan->span->pvt;
+	struct b4xxp_span *bspan = container_of(chan->span, struct b4xxp_span, span);
 	int channel;
 
 	memset(ec, 0, sizeof(*ec));
@@ -2138,7 +2205,7 @@ static int b4xxp_ioctl(struct dahdi_chan *chan, unsigned int cmd, unsigned long
 
 static int b4xxp_startup(struct dahdi_span *span)
 {
-	struct b4xxp_span *bspan = span->pvt;
+	struct b4xxp_span *bspan = container_of(span, struct b4xxp_span, span);
 	struct b4xxp *b4 = bspan->parent;
 
 	if (!b4->running)
@@ -2149,7 +2216,7 @@ static int b4xxp_startup(struct dahdi_span *span)
 
 static int b4xxp_shutdown(struct dahdi_span *span)
 {
-	struct b4xxp_span *bspan = span->pvt;
+	struct b4xxp_span *bspan = container_of(span, struct b4xxp_span, span);
 
 	hfc_disable_interrupts(bspan->parent);
 	return 0;
@@ -2172,18 +2239,19 @@ static void b4xxp_reset_span(struct b4xxp_span *bspan)
 static int b4xxp_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
 {
 	int i;
-	struct b4xxp_span *bspan = span->pvt;
+	struct b4xxp_span *bspan = container_of(span, struct b4xxp_span, span);
 	struct b4xxp *b4 = bspan->parent;
 
 	if (DBG)
-		dev_info(b4->dev, "Configuring span %d\n", span->spanno);
+		dev_info(b4->dev, "Configuring span %d offset %d to be sync %d\n", span->spanno, span->offset, lc->sync);
 
 #if 0
-	if (lc->sync > 0 && bspan->te_mode) {
+	if (lc->sync > 0 && !bspan->te_mode) {
 		dev_info(b4->dev, "Span %d is not in NT mode, removing from sync source list\n", span->spanno);
 		lc->sync = 0;
 	}
 #endif
+
 	if (lc->sync < 0 || lc->sync > 4) {
 		dev_info(b4->dev, "Span %d has invalid sync priority (%d), removing from sync source list\n", span->spanno, lc->sync);
 		lc->sync = 0;
@@ -2191,7 +2259,7 @@ static int b4xxp_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc
 
 	/* remove this span number from the current sync sources, if there */
 	for (i = 0; i < b4->numspans; i++) {
-		if (b4->spans[i].sync == span->spanno) {
+		if (b4->spans[i].sync == (span->offset + 1)) {
 			b4->spans[i].sync = 0;
 		}
 	}
@@ -2199,7 +2267,7 @@ static int b4xxp_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc
 	/* if a sync src, put it in proper place */
 	b4->spans[span->offset].syncpos = lc->sync;
 	if (lc->sync) {
-		b4->spans[lc->sync - 1].sync = span->spanno;
+		b4->spans[lc->sync - 1].sync = (span->offset + 1);
 	}
 
 	b4xxp_reset_span(bspan);
@@ -2263,7 +2331,7 @@ static int b4xxp_open(struct dahdi_chan *chan)
 	if (DBG_FOPS && DBG_SPANFILTER)
 		dev_info(b4->dev, "open() on chan %s (%i/%i)\n", chan->name, chan->channo, chan->chanpos);
 
-	hfc_reset_fifo_pair(b4, bspan->fifos[chan->chanpos], 0, 0);
+	hfc_reset_fifo_pair(b4, bspan->fifos[chan->chanpos - 1], 0, 0);
 	return 0;
 }
 
@@ -2276,7 +2344,7 @@ static int b4xxp_close(struct dahdi_chan *chan)
 	if (DBG_FOPS && DBG_SPANFILTER)
 		dev_info(b4->dev, "close() on chan %s (%i/%i)\n", chan->name, chan->channo, chan->chanpos);
 
-	hfc_reset_fifo_pair(b4, bspan->fifos[chan->chanpos], 1, 1);
+	hfc_reset_fifo_pair(b4, bspan->fifos[chan->chanpos - 1], 1, 1);
 	return 0;
 }
 
@@ -2302,6 +2370,19 @@ static void b4xxp_hdlc_hard_xmit(struct dahdi_chan *chan)
 
 /* internal functions, not specific to the hardware or DAHDI */
 
+static const struct dahdi_span_ops b4xxp_span_ops = {
+	.owner = THIS_MODULE,
+	.spanconfig = b4xxp_spanconfig,
+	.chanconfig = b4xxp_chanconfig,
+	.startup = b4xxp_startup,
+	.shutdown = b4xxp_shutdown,
+	.open = b4xxp_open,
+	.close  = b4xxp_close,
+	.ioctl = b4xxp_ioctl,
+	.hdlc_hard_xmit = b4xxp_hdlc_hard_xmit,
+	.echocan_create = b4xxp_echocan_create,
+};
+
 /* initialize the span/chan structures. Doesn't touch hardware, although the callbacks might. */
 static void init_spans(struct b4xxp *b4)
 {
@@ -2315,13 +2396,15 @@ static void init_spans(struct b4xxp *b4)
 		bspan->parent = b4;
 
 		bspan->span.irq = b4->pdev->irq;
-		bspan->span.pvt = bspan;
 		bspan->span.spantype = (bspan->te_mode) ? "TE" : "NT";
 		bspan->span.offset = i;
 		bspan->span.channels = WCB4XXP_CHANNELS_PER_SPAN;
 		bspan->span.flags = 0;
 
-		bspan->span.deflaw = DAHDI_LAW_ALAW;
+		if (!strcasecmp(companding, "ulaw"))
+			bspan->span.deflaw = DAHDI_LAW_MULAW;
+		else
+			bspan->span.deflaw = DAHDI_LAW_ALAW;
 		/* For simplicty, we'll accept all line modes since BRI
 		 * ignores this setting anyway.*/
 		bspan->span.linecompat = DAHDI_CONFIG_AMI | 
@@ -2336,18 +2419,7 @@ static void init_spans(struct b4xxp *b4)
 		sprintf(bspan->span.location, "PCI Bus %02d Slot %02d",
 			b4->pdev->bus->number, PCI_SLOT(b4->pdev->devfn) + 1);
 
-		bspan->span.owner = THIS_MODULE;
-		bspan->span.spanconfig = b4xxp_spanconfig;
-		bspan->span.chanconfig = b4xxp_chanconfig;
-		bspan->span.startup = b4xxp_startup;
-		bspan->span.shutdown = b4xxp_shutdown;
-		bspan->span.open = b4xxp_open;
-		bspan->span.close  = b4xxp_close;
-		bspan->span.ioctl = b4xxp_ioctl;
-		bspan->span.hdlc_hard_xmit = b4xxp_hdlc_hard_xmit;
-		if (vpmsupport && CARD_HAS_EC(b4))
-			bspan->span.echocan_create = echocan_create;
-
+		bspan->span.ops = &b4xxp_span_ops;
 /* HDLC stuff */
 		bspan->sigchan = NULL;
 		bspan->sigactive = 0;
@@ -2863,6 +2935,7 @@ static void __devexit b4xxp_remove(struct pci_dev *pdev)
 		}
 
 		b4xxp_init_stage1(b4);
+		remove_sysfs_files(b4);
 		free_irq(pdev->irq, b4);
 		pci_set_drvdata(pdev, NULL);
 		pci_iounmap(pdev, b4->ioaddr);
@@ -2881,13 +2954,14 @@ static void __devexit b4xxp_remove(struct pci_dev *pdev)
 	return;
 }
 
-static struct pci_device_id b4xx_ids[] __devinitdata =
+static DEFINE_PCI_DEVICE_TABLE(b4xx_ids) =
 {
 	{ 0xd161, 0xb410, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&wcb4xxp },
 	{ 0x1397, 0x16b8, 0x1397, 0xb552, 0, 0, (unsigned long)&hfc8s },
 	{ 0x1397, 0x16b8, 0x1397, 0xb55b, 0, 0, (unsigned long)&hfc8s },
 	{ 0x1397, 0x08b4, 0x1397, 0xb520, 0, 0, (unsigned long)&hfc4s },
 	{ 0x1397, 0x08b4, 0x1397, 0xb550, 0, 0, (unsigned long)&hfc4s },
+	{ 0x1397, 0x08b4, 0x1397, 0xb752, 0, 0, (unsigned long)&hfc4s },
 	{ 0x1397, 0x08b4, 0x1397, 0xb556, 0, 0, (unsigned long)&hfc2s },
 	{ 0x1397, 0x08b4, 0x1397, 0xe884, 0, 0, (unsigned long)&hfc2s_OV },
 	{ 0x1397, 0x08b4, 0x1397, 0xe888, 0, 0, (unsigned long)&hfc4s_OV },
@@ -2946,6 +3020,7 @@ module_param(alarmdebounce, int, S_IRUGO | S_IWUSR);
 module_param(vpmsupport, int, S_IRUGO);
 module_param(timer_1_ms, int, S_IRUGO | S_IWUSR);
 module_param(timer_3_ms, int, S_IRUGO | S_IWUSR);
+module_param(companding, charp, S_IRUGO);
 
 MODULE_PARM_DESC(debug, "bitmap: 1=general 2=dtmf 4=regops 8=fops 16=ec 32=st state 64=hdlc 128=alarm");
 MODULE_PARM_DESC(spanfilter, "debug filter for spans. bitmap: 1=port 1, 2=port 2, 4=port 3, 8=port 4");
@@ -2959,6 +3034,8 @@ MODULE_PARM_DESC(alarmdebounce, "msec to wait before set/clear alarm condition")
 MODULE_PARM_DESC(vpmsupport, "1=enable hardware EC, 0=disable hardware EC");
 MODULE_PARM_DESC(timer_1_ms, "NT: msec to wait for link activation, TE: unused.");
 MODULE_PARM_DESC(timer_3_ms, "TE: msec to wait for link activation, NT: unused.");
+MODULE_PARM_DESC(companding, "Change the companding to \"alaw\" or \"ulaw\""\
+				"(alaw by default)");
 
 MODULE_AUTHOR("Digium Incorporated <support@digium.com>");
 MODULE_DESCRIPTION("B410P & Similars multi-port BRI module driver.");
