@@ -550,7 +550,7 @@ struct b400m_span {
 	int fifos[B400M_CHANNELS_PER_SPAN];			/* B1, B2, D <--> host fifo numbers */
 
 	/* HDLC controller fields */
-	struct dahdi_span *span;		/* pointer to the actual dahdi_span */
+	struct wctdm_span *wspan;  /* pointer to the actual dahdi_span */
 	struct dahdi_chan *sigchan;		/* pointer to the signalling channel for this span */
 	int sigactive;				/* nonzero means we're in the middle of sending an HDLC frame */
 	atomic_t hdlc_pending;			/* hdlc_hard_xmit() increments, hdlc_tx_frame() decrements */
@@ -604,10 +604,10 @@ static void hfc_start_st(struct b400m_span *s);
 static void hfc_stop_st(struct b400m_span *s);
 
 void b400m_set_dahdi_span(struct b400m *b4, int spanno,
-			  struct dahdi_span *span)
+			  struct wctdm_span *wspan)
 {
-	span->pvt = &b4->spans[spanno];
-	b4->spans[spanno].span = span;
+	b4->spans[spanno].wspan = wspan;
+	wspan->bspan = &b4->spans[spanno];
 }
 
 static inline void flush_hw(void)
@@ -1446,11 +1446,11 @@ static void hfc_update_st_timers(struct b400m *b4)
 				hfc_timer_expire(s, j);
 		}
 
-		if (s->span && s->newalarm != s->span->alarms &&
+		if (s->wspan && s->newalarm != s->wspan->span.alarms &&
 		    time_after_eq(b4->ticks, s->alarmtimer)) {
-			s->span->alarms = s->newalarm;
+			s->wspan->span.alarms = s->newalarm;
 			if ((!s->newalarm && bri_teignorered) || (!bri_teignorered))
-				dahdi_alarm_notify(s->span);
+				dahdi_alarm_notify(&s->wspan->span);
 
 			if (DBG_ALARM) {
 				dev_info(&b4->wc->vb.pdev->dev, "span %d: alarm " \
@@ -1610,9 +1610,9 @@ static void hfc_reset_st(struct b400m_span *s)
 	b400m_setreg_ra(b4, R_SU_SEL, s->port, A_SU_WR_STA, V_SU_LD_STA);
 	flush_hw();			/* make sure write hit hardware */
 
-	s->span->alarms = DAHDI_ALARM_RED;
+	s->wspan->span.alarms = DAHDI_ALARM_RED;
 	s->newalarm = DAHDI_ALARM_RED;
-	dahdi_alarm_notify(s->span);
+	dahdi_alarm_notify(&s->wspan->span);
 
 	/* set up the clock control register.  Must be done before we activate
 	 * the interface. */
@@ -2058,9 +2058,14 @@ static void xhfc_init_stage2(struct b400m *b4)
 	 */
 }
 
+static inline struct b400m_span *bspan_from_dspan(struct dahdi_span *span)
+{
+	return container_of(span, struct wctdm_span, span)->bspan;
+}
+
 static int xhfc_startup(struct dahdi_span *span)
 {
-	struct b400m_span *bspan = span->pvt;
+	struct b400m_span *bspan = bspan_from_dspan(span);
 	struct b400m *b4 = bspan->parent;
 	if (!b4->running)
 		hfc_enable_interrupts(bspan->parent);
@@ -2141,6 +2146,8 @@ static int b400m_set_ntte(struct b400m_span *bspan, int te_mode, int term_on)
 	int all_modes = 0, all_terms = 0;
 	int i;
 
+	bspan->wspan->span.spantype = (te_mode > 0) ? "TE" : "NT";
+
 	bspan->te_mode = te_mode;
 	bspan->term_on = term_on;
 
@@ -2177,7 +2184,7 @@ int b400m_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
 	int te_mode, term;
 	int pos;
 
-	bspan = span->pvt;
+	bspan = bspan_from_dspan(span);
 	b4 = bspan->parent;
 	wc = b4->wc;
 
@@ -2218,7 +2225,7 @@ int b400m_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
 
 	wc->spans[pos]->timing_priority = lc->sync;
 
-	bspan->span = span;
+	bspan->wspan = container_of(span, struct wctdm_span, span);
 	xhfc_reset_span(bspan);
 
 	/* call startup() manually here, because DAHDI won't call the startup
@@ -2247,11 +2254,11 @@ int b400m_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
 int b400m_chanconfig(struct dahdi_chan *chan, int sigtype)
 {
 	int alreadyrunning;
-	struct b400m_span *bspan = chan->span->pvt;
+	struct b400m_span *bspan = bspan_from_dspan(chan->span);
 	struct b400m *b4 = bspan->parent;
 	int res;
 
-	alreadyrunning = bspan->span->flags & DAHDI_FLAG_RUNNING;
+	alreadyrunning = bspan->wspan->span.flags & DAHDI_FLAG_RUNNING;
 
 	if (DBG_FOPS) {
 		b4_info(b4, "%s channel %d (%s) sigtype %08x\n",
@@ -2294,7 +2301,7 @@ int b400m_dchan(struct dahdi_span *span)
 	int res;
 	int i;
 
-	bspan = span->pvt;
+	bspan = bspan_from_dspan(span);
 	b4 = bspan->parent;
 #ifdef HARDHDLC_RX
 	return 0;
@@ -2518,7 +2525,7 @@ void wctdm_hdlc_hard_xmit(struct dahdi_chan *chan)
 	int span;
 
 	dspan = chan->span;
-	bspan = dspan->pvt;
+	bspan = bspan_from_dspan(dspan);
 	b4 = bspan->parent;
 	wc = b4->wc;
 	span = bspan->port;
@@ -2569,8 +2576,8 @@ static int b400m_probe(struct wctdm *wc, int modpos)
 	}
 
 	spin_lock_init(&b4->reglock);
-	init_MUTEX(&b4->regsem);
-	init_MUTEX(&b4->fifosem);
+	sema_init(&b4->regsem, 1);
+	sema_init(&b4->fifosem, 1);
 
 	for (x = 0; x < 4; x++) {
 		fasthdlc_init(&b4->spans[x].rxhdlc, FASTHDLC_MODE_16);
@@ -2718,8 +2725,8 @@ void wctdm_unload_b400m(struct wctdm *wc, int card)
 				wc->spans[i]->timing_priority = 0;
 		}
 
-		for (i = 0; i < 4; i++)
-			b4->spans[i].span->flags &= ~DAHDI_FLAG_RUNNING;
+		for (i = 0; i < ARRAY_SIZE(b4->spans); i++)
+			b4->spans[i].wspan->span.flags &= ~DAHDI_FLAG_RUNNING;
 
 		wctdm_change_card_sync_src(b4->wc, 0, 0);
 
