@@ -31,7 +31,7 @@
 #include "dahdi_debug.h"
 #include "xbus-core.h"
 
-static const char rcsid[] = "$Id: card_fxo.c 7883 2010-01-12 10:08:32Z tzafrir $";
+static const char rcsid[] = "$Id: card_fxo.c 8986 2010-07-26 00:30:43Z sruffell $";
 
 static DEF_PARM(int, debug, 0, 0644, "Print DBG statements");
 static DEF_PARM(uint, poll_battery_interval, 500, 0644, "Poll battery interval in milliseconds (0 - disable)");
@@ -87,10 +87,12 @@ static /* 0x0F */ DECLARE_CMD(FXO, XPD_STATE, bool on);
 
 static bool fxo_packet_is_valid(xpacket_t *pack);
 static void fxo_packet_dump(const char *msg, xpacket_t *pack);
+#ifdef CONFIG_PROC_FS
 static int proc_fxo_info_read(char *page, char **start, off_t off, int count, int *eof, void *data);
 #ifdef	WITH_METERING
 static int proc_xpd_metering_read(char *page, char **start, off_t off, int count, int *eof, void *data);
 #endif
+#endif
 static void dahdi_report_battery(xpd_t *xpd, lineno_t chan);
 
 #define	PROC_REGISTER_FNAME	"slics"
@@ -403,7 +405,8 @@ static int fxo_proc_create(xbus_t *xbus, xpd_t *xpd)
 	priv->fxo_info = create_proc_read_entry(PROC_FXO_INFO_FNAME, 0444, xpd->proc_xpd_dir, proc_fxo_info_read, xpd);
 	if(!priv->fxo_info) {
 		XPD_ERR(xpd, "Failed to create proc file '%s'\n", PROC_FXO_INFO_FNAME);
-		goto err;
+		fxo_proc_remove(xbus, xpd);
+		return -EINVAL;
 	}
 	SET_PROC_DIRENTRY_OWNER(priv->fxo_info);
 #ifdef	WITH_METERING
@@ -412,14 +415,13 @@ static int fxo_proc_create(xbus_t *xbus, xpd_t *xpd)
 			proc_xpd_metering_read, xpd);
 	if(!priv->meteringfile) {
 		XPD_ERR(xpd, "Failed to create proc file '%s'\n", PROC_METERING_FNAME);
-		goto err;
+		fxo_proc_remove(xbus, xpd);
+		return -EINVAL;
 	}
 	SET_PROC_DIRENTRY_OWNER(priv->meteringfile);
 #endif
 #endif
 	return 0;
-err:
-	return -EINVAL;
 }
 
 static xpd_t *FXO_card_new(xbus_t *xbus, int unit, int subunit, const xproto_table_t *proto_table,
@@ -509,8 +511,6 @@ static int FXO_card_dahdi_preregistration(xpd_t *xpd, bool on)
 	BUG_ON(!priv);
 	timer_count = xpd->timer_count;
 	XPD_DBG(GENERAL, xpd, "%s\n", (on)?"ON":"OFF");
-	xpd->span.owner = THIS_MODULE;
-	xpd->span.spantype = "FXO";
 	for_each_line(xpd, i) {
 		struct dahdi_chan	*cur_chan = XPD_CHAN(xpd, i);
 
@@ -1131,6 +1131,7 @@ static xproto_table_t PROTO_TABLE(FXO) = {
 		.card_pcm_recompute	= generic_card_pcm_recompute,
 		.card_pcm_fromspan	= generic_card_pcm_fromspan,
 		.card_pcm_tospan	= generic_card_pcm_tospan,
+		.card_timing_priority	= generic_timing_priority,
 		.card_ioctl	= FXO_card_ioctl,
 		.card_open	= FXO_card_open,
 		.card_register_reply	= FXO_card_register_reply,
@@ -1157,6 +1158,7 @@ static void fxo_packet_dump(const char *msg, xpacket_t *pack)
 
 /*------------------------- DAA Handling --------------------------*/
 
+#ifdef	CONFIG_PROC_FS
 static int proc_fxo_info_read(char *page, char **start, off_t off, int count, int *eof, void *data)
 {
 	int			len = 0;
@@ -1273,6 +1275,7 @@ static int proc_fxo_info_read(char *page, char **start, off_t off, int count, in
 		len = 0;
 	return len;
 }
+#endif
 
 #ifdef	WITH_METERING
 static int proc_xpd_metering_read(char *page, char **start, off_t off, int count, int *eof, void *data)
