@@ -31,21 +31,16 @@
 #include "dahdi_debug.h"
 #include "xbus-core.h"
 
-static const char rcsid[] = "$Id: card_fxs.c 6794 2009-06-29 20:56:07Z sruffell $";
+static const char rcsid[] = "$Id: card_fxs.c 8986 2010-07-26 00:30:43Z sruffell $";
 
 static DEF_PARM(int, debug, 0, 0644, "Print DBG statements");	/* must be before dahdi_debug.h */
 static DEF_PARM_BOOL(reversepolarity, 0, 0644, "Reverse Line Polarity");
-static DEF_PARM_BOOL(vmwineon, 0, 0644, "Indicate voicemail to a neon lamp");
 static DEF_PARM_BOOL(dtmf_detection, 1, 0644, "Do DTMF detection in hardware");
 #ifdef	POLL_DIGITAL_INPUTS
 static DEF_PARM(uint, poll_digital_inputs, 1000, 0644, "Poll Digital Inputs");
 #endif
 
-#ifdef	DAHDI_VMWI
-static DEF_PARM_BOOL(vmwi_ioctl, 0, 0644, "Asterisk support VMWI notification via ioctl");
-#else
-#define	vmwi_ioctl	0	/* not supported */
-#endif
+static DEF_PARM_BOOL(vmwi_ioctl, 1, 0644, "Asterisk support VMWI notification via ioctl");
 
 /* Signaling is opposite (fxo signalling for fxs card) */
 #if 1
@@ -54,6 +49,10 @@ static DEF_PARM_BOOL(vmwi_ioctl, 0, 0644, "Asterisk support VMWI notification vi
 #define	FXS_DEFAULT_SIGCAP	(DAHDI_SIG_SF | DAHDI_SIG_EM)
 #endif
 
+#define	VMWI_TYPE(priv, pos, type)	\
+	((priv)->vmwisetting[pos].vmwi_type & DAHDI_VMWI_ ## type)
+#define	VMWI_NEON(priv, pos)		VMWI_TYPE(priv, pos, HVAC)
+
 #define	LINES_DIGI_OUT	2
 #define	LINES_DIGI_INP	4
 
@@ -109,10 +108,12 @@ static /* 0x0F */ DECLARE_CMD(FXS, XPD_STATE, bool on);
 
 static bool fxs_packet_is_valid(xpacket_t *pack);
 static void fxs_packet_dump(const char *msg, xpacket_t *pack);
+#ifdef CONFIG_PROC_FS
 static int proc_fxs_info_read(char *page, char **start, off_t off, int count, int *eof, void *data);
 #ifdef	WITH_METERING
 static int proc_xpd_metering_write(struct file *file, const char __user *buffer, unsigned long count, void *data);
 #endif
+#endif
 static void start_stop_vm_led(xbus_t *xbus, xpd_t *xpd, lineno_t pos);
 
 #define	PROC_REGISTER_FNAME	"slics"
@@ -140,6 +141,7 @@ struct FXS_priv_data {
 #define OHT_TIMER		6000	/* How long after RING to retain OHT */
 	enum fxs_state		idletxhookstate[CHANNELS_PERXPD];	/* IDLE changing hook state */
 	enum fxs_state		lasttxhook[CHANNELS_PERXPD];
+	struct dahdi_vmwi_info	vmwisetting[CHANNELS_PERXPD];
 };
 
 /*
@@ -181,7 +183,7 @@ static void vmwi_search(xpd_t *xpd, lineno_t pos, bool on)
 
 	priv = xpd->priv;
 	BUG_ON(!xpd);
-	if(vmwineon && on) {
+	if (VMWI_NEON(priv, pos) && on) {
 		LINE_DBG(SIGNAL, xpd, pos, "START\n");
 		BIT_SET(priv->search_fsk_pattern, pos);
 	} else {
@@ -356,7 +358,8 @@ static int fxs_proc_create(xbus_t *xbus, xpd_t *xpd)
 	priv->fxs_info = create_proc_read_entry(PROC_FXS_INFO_FNAME, 0444, xpd->proc_xpd_dir, proc_fxs_info_read, xpd);
 	if(!priv->fxs_info) {
 		XPD_ERR(xpd, "Failed to create proc file '%s'\n", PROC_FXS_INFO_FNAME);
-		goto err;
+		fxs_proc_remove(xbus, xpd);
+		return -EINVAL;
 	}
 	SET_PROC_DIRENTRY_OWNER(priv->fxs_info);
 #ifdef	WITH_METERING
@@ -364,7 +367,8 @@ static int fxs_proc_create(xbus_t *xbus, xpd_t *xpd)
 	priv->meteringfile = create_proc_entry(PROC_METERING_FNAME, 0200, xpd->proc_xpd_dir);
 	if(!priv->meteringfile) {
 		XPD_ERR(xpd, "Failed to create proc file '%s'\n", PROC_METERING_FNAME);
-		goto err;
+		fxs_proc_remove(xbus, xpd);
+		return -EINVAL;
 	}
 	SET_PROC_DIRENTRY_OWNER(priv->meteringfile);
 	priv->meteringfile->write_proc = proc_xpd_metering_write;
@@ -373,8 +377,6 @@ static int fxs_proc_create(xbus_t *xbus, xpd_t *xpd)
 #endif
 #endif
 	return 0;
-err:
-	return -EINVAL;
 }
 
 static xpd_t *FXS_card_new(xbus_t *xbus, int unit, int subunit, const xproto_table_t *proto_table,
@@ -494,7 +496,6 @@ static int FXS_card_dahdi_preregistration(xpd_t *xpd, bool on)
 	priv = xpd->priv;
 	BUG_ON(!priv);
 	XPD_DBG(GENERAL, xpd, "%s\n", (on)?"on":"off");
-	xpd->span.owner = THIS_MODULE;
 	xpd->span.spantype = "FXS";
 	for_each_line(xpd, i) {
 		struct dahdi_chan	*cur_chan = XPD_CHAN(xpd, i);
@@ -513,6 +514,10 @@ static int FXS_card_dahdi_preregistration(xpd_t *xpd, bool on)
 		cur_chan->chanpos = i + 1;
 		cur_chan->pvt = xpd;
 		cur_chan->sigcap = FXS_DEFAULT_SIGCAP;
+		if (!vmwi_ioctl) {
+			/* Old asterisk, assume default VMWI type */
+			priv->vmwisetting[i].vmwi_type = DAHDI_VMWI_HVAC;
+		}
 	}
 	for_each_line(xpd, i) {
 		MARK_ON(priv, i, LED_GREEN);
@@ -557,13 +562,16 @@ static void __do_mute_dtmf(xpd_t *xpd, int pos, bool muteit)
 	CALL_XMETHOD(card_pcm_recompute, xpd->xbus, xpd, 0);	/* already spinlocked */
 }
 
-static int set_vm_led_mode(xbus_t *xbus, xpd_t *xpd, int pos, bool msg_waiting)
+static int set_vm_led_mode(xbus_t *xbus, xpd_t *xpd, int pos,
+		unsigned int msg_waiting)
 {
 	int	ret = 0;
+	struct FXS_priv_data	*priv;
 	BUG_ON(!xbus);
 	BUG_ON(!xpd);
 
-	if (vmwineon && msg_waiting) {
+	priv = xpd->priv;
+	if (VMWI_NEON(priv, pos) && msg_waiting) {
 		/* A write to register 0x40 will now turn on/off the VM led */
 		LINE_DBG(SIGNAL, xpd, pos, "NEON\n");
 		ret += SLIC_INDIRECT_REQUEST(xbus, xpd, pos, SLIC_WRITE, 0x16, 0xE8, 0x03);
@@ -596,17 +604,18 @@ static int set_vm_led_mode(xbus_t *xbus, xpd_t *xpd, int pos, bool msg_waiting)
 static void start_stop_vm_led(xbus_t *xbus, xpd_t *xpd, lineno_t pos)
 {
 	struct FXS_priv_data	*priv;
-	bool		on;
+	unsigned int		msgs;
 
 	BUG_ON(!xpd);
 	if (IS_SET(xpd->digital_outputs | xpd->digital_inputs, pos))
 		return;
 	priv = xpd->priv;
-	on = IS_SET(xpd->msg_waiting, pos);
-	LINE_DBG(SIGNAL, xpd, pos, "%s\n", (on)?"ON":"OFF");
-	set_vm_led_mode(xbus, xpd, pos, on);
-	do_chan_power(xbus, xpd, pos, on);
-	linefeed_control(xbus, xpd, pos, (on) ? FXS_LINE_RING : priv->idletxhookstate[pos]);
+	msgs = xpd->msg_waiting[pos];
+	LINE_DBG(SIGNAL, xpd, pos, "%s\n", (msgs) ? "ON" : "OFF");
+	set_vm_led_mode(xbus, xpd, pos, msgs);
+	do_chan_power(xbus, xpd, pos, msgs > 0);
+	linefeed_control(xbus, xpd, pos, (msgs > 0) ?
+			FXS_LINE_RING : priv->idletxhookstate[pos]);
 }
 
 static int relay_out(xpd_t *xpd, int pos, bool on)
@@ -761,6 +770,47 @@ static int FXS_card_hooksig(xbus_t *xbus, xpd_t *xpd, int pos, enum dahdi_txsig
 	return ret;
 }
 
+static int set_vmwi(xpd_t *xpd, int pos, unsigned long arg)
+{
+	struct FXS_priv_data	*priv;
+	struct dahdi_vmwi_info	vmwisetting;
+	const int vmwi_flags = DAHDI_VMWI_LREV | DAHDI_VMWI_HVDC
+		| DAHDI_VMWI_HVAC;
+
+	priv = xpd->priv;
+	BUG_ON(!priv);
+	if (copy_from_user(&vmwisetting, (__user void *)arg,
+				sizeof(vmwisetting)))
+		return -EFAULT;
+	if ((vmwisetting.vmwi_type & ~vmwi_flags) != 0) {
+		LINE_NOTICE(xpd, pos, "Bad DAHDI_VMWI_CONFIG: 0x%X\n",
+			vmwisetting.vmwi_type);
+		return -EINVAL;
+	}
+	LINE_DBG(SIGNAL, xpd, pos,
+			"DAHDI_VMWI_CONFIG: 0x%X\n",
+			vmwisetting.vmwi_type);
+	if (VMWI_TYPE(priv, pos, LREV)) {
+		LINE_NOTICE(xpd, pos,
+			"%s: VMWI(lrev) is not implemented yet. Ignored.\n",
+			__func__);
+	}
+	if (VMWI_TYPE(priv, pos, HVDC)) {
+		LINE_NOTICE(xpd, pos,
+			"%s: VMWI(hvdc) is not implemented yet. Ignored.\n",
+			__func__);
+	}
+	if (VMWI_TYPE(priv, pos, HVAC)) {
+		;	/* VMWI_NEON */
+	}
+	if (priv->vmwisetting[pos].vmwi_type == 0) {
+		;	/* Disable VMWI */
+	}
+	priv->vmwisetting[pos] = vmwisetting;
+	set_vm_led_mode(xpd->xbus, xpd, pos, xpd->msg_waiting[pos]);
+	return 0;
+}
+
 /*
  * Private ioctl()
  * We don't need it now, since we detect vmwi via FSK patterns
@@ -800,7 +850,7 @@ static int FXS_card_ioctl(xpd_t *xpd, int pos, unsigned int cmd, unsigned long a
 				CALL_XMETHOD(card_pcm_recompute, xbus, xpd, priv->search_fsk_pattern);
 				LINE_DBG(SIGNAL, xpd, pos, "Start OHT_TIMER. wanted_pcm_mask=0x%X\n", xpd->wanted_pcm_mask);
 			}
-			if(vmwineon && !IS_OFFHOOK(xpd, pos))
+			if (VMWI_NEON(priv, pos) && !IS_OFFHOOK(xpd, pos))
 				start_stop_vm_led(xbus, xpd, pos);
 			return 0;
 		case DAHDI_TONEDETECT:
@@ -869,7 +919,10 @@ static int FXS_card_ioctl(xpd_t *xpd, int pos, unsigned int cmd, unsigned long a
 				priv->lasttxhook[pos] &= ~FXS_LINE_RING;
 			linefeed_control(xbus, xpd, pos, priv->lasttxhook[pos]);
 			return 0;
-#ifdef	DAHDI_VMWI
+		case DAHDI_VMWI_CONFIG:
+			if (set_vmwi(xpd, pos, arg) < 0)
+				return -EINVAL;
+			return 0;
 		case DAHDI_VMWI:		/* message-waiting led control */
 			if (get_user(val, (int __user *)arg))
 				return -EFAULT;
@@ -884,12 +937,10 @@ static int FXS_card_ioctl(xpd_t *xpd, int pos, unsigned int cmd, unsigned long a
 			/* Digital inputs/outputs don't have VM leds */
 			if (IS_SET(xpd->digital_inputs | xpd->digital_outputs, pos))
 				return 0;
-			if (val)
-				BIT_SET(xpd->msg_waiting, pos);
-			else
-				BIT_CLR(xpd->msg_waiting, pos);
+			xpd->msg_waiting[pos] = val;
+			LINE_DBG(SIGNAL, xpd, pos, "DAHDI_VMWI: %s\n",
+					(val) ? "yes" : "no");
 			return 0;
-#endif
 		default:
 			report_bad_ioctl(THIS_MODULE->name, xpd, pos, cmd);
 	}
@@ -1046,11 +1097,11 @@ static void detect_vmwi(xpd_t *xpd)
 			oht_pcm(xpd, i, 0);
 			if(unlikely(mem_equal(writechunk, FSK_ON_PATTERN, DAHDI_CHUNKSIZE))) {
 				LINE_DBG(SIGNAL, xpd, i, "MSG WAITING ON\n");
-				BIT_SET(xpd->msg_waiting, i);
+				xpd->msg_waiting[i] = 1;
 				start_stop_vm_led(xbus, xpd, i);
 			} else if(unlikely(mem_equal(writechunk, FSK_OFF_PATTERN, DAHDI_CHUNKSIZE))) {
 				LINE_DBG(SIGNAL, xpd, i, "MSG WAITING OFF\n");
-				BIT_CLR(xpd->msg_waiting, i);
+				xpd->msg_waiting[i] = 0;
 				start_stop_vm_led(xbus, xpd, i);
 			} else {
 				int	j;
@@ -1100,7 +1151,7 @@ static int FXS_card_tick(xbus_t *xbus, xpd_t *xpd)
 		}
 	}
 	if(SPAN_REGISTERED(xpd)) {
-		if(vmwineon && !vmwi_ioctl && priv->search_fsk_pattern)
+		if (!vmwi_ioctl && priv->search_fsk_pattern)
 			detect_vmwi(xpd);	/* Detect via FSK modulation */
 	}
 	return 0;
@@ -1363,6 +1414,7 @@ static xproto_table_t PROTO_TABLE(FXS) = {
 		.card_pcm_recompute	= generic_card_pcm_recompute,
 		.card_pcm_fromspan	= generic_card_pcm_fromspan,
 		.card_pcm_tospan	= generic_card_pcm_tospan,
+		.card_timing_priority	= generic_timing_priority,
 		.card_open	= FXS_card_open,
 		.card_close	= FXS_card_close,
 		.card_ioctl	= FXS_card_ioctl,
@@ -1390,6 +1442,7 @@ static void fxs_packet_dump(const char *msg, xpacket_t *pack)
 
 /*------------------------- SLIC Handling --------------------------*/
 
+#ifdef	CONFIG_PROC_FS
 static int proc_fxs_info_read(char *page, char **start, off_t off, int count, int *eof, void *data)
 {
 	int			len = 0;
@@ -1458,6 +1511,7 @@ static int proc_fxs_info_read(char *page, char **start, off_t off, int count, in
 		len = 0;
 	return len;
 }
+#endif
 
 #ifdef	WITH_METERING
 static int proc_xpd_metering_write(struct file *file, const char __user *buffer, unsigned long count, void *data)
@@ -1545,11 +1599,7 @@ static int __init card_fxs_startup(void)
 #else
 	INFO("FEATURE: without DIGITAL INPUTS support\n");
 #endif
-#ifdef	DAHDI_VMWI
-	INFO("FEATURE: DAHDI_VMWI\n");
-#else
-	INFO("FEATURE: NO DAHDI_VMWI\n");
-#endif
+	INFO("FEATURE: DAHDI_VMWI (HVAC only)\n");
 #ifdef	WITH_METERING
 	INFO("FEATURE: WITH METERING Generation\n");
 #else
