@@ -33,7 +33,7 @@
 #include "dahdi_debug.h"
 #include "parport_debug.h"
 
-static const char rcsid[] = "$Id: xbus-pcm.c 6641 2009-05-19 16:01:27Z tzafrir $";
+static const char rcsid[] = "$Id: xbus-pcm.c 8984 2010-07-26 00:30:39Z sruffell $";
 
 extern int debug;
 #ifdef	OPTIMIZE_CHANMUTE
@@ -56,13 +56,20 @@ static struct xpp_ticker	dahdi_ticker;
  */
 static struct xpp_ticker	*ref_ticker = NULL;
 static spinlock_t		ref_ticker_lock = SPIN_LOCK_UNLOCKED;
+static spinlock_t		elect_syncer_lock = SPIN_LOCK_UNLOCKED;
 static bool			force_dahdi_sync = 0;	/* from /sys/bus/astribanks/drivers/xppdrv/sync */
 static xbus_t			*global_ticker;
 static struct xpp_ticker	global_ticks_series;
 
 #define	PROC_SYNC		"sync"
+#define SYNC_CYCLE		500	/* Sampling cycle in usec */
+#define SYNC_CYCLE_SAMPLE 	100	/* Samples from end of SYNC_CYCLE */
+#define SYNC_CONVERGE		10	/* Number of SYNC_CYCLE's to converge speed */
+#define SYNC_CENTER		500	/* Offset from ref_ticker to other AB's */
+#define SYNC_DELTA		40	/* If within +/-SYNC_DELTA, try to stay there */
 #define	BIG_TICK_INTERVAL	1000
-#define	SYNC_ADJ_MAX		63	/* maximal firmware drift unit (63) */
+#define	SYNC_ADJ_MAX		20	/* maximal firmware drift unit (hardware limit 63) */
+
 /*
  * The USB bulk endpoints have a large jitter in the timing of frames
  * from the AB to the ehci-hcd. This is because we cannot predict
@@ -123,12 +130,6 @@ static int xpp_ticker_step(struct xpp_ticker *ticker, const struct timeval *t)
 	return cycled;
 }
 
-static inline void driftinfo_recalc(struct xpp_drift *driftinfo)
-{
-	driftinfo->delta_max = INT_MIN;
-	driftinfo->delta_min = INT_MAX;
-}
-
 /*
  * No locking. It is called only from:
  *   - update_sync_master() in a globall spinlock protected code.
@@ -136,11 +137,12 @@ static inline void driftinfo_recalc(struct xpp_drift *driftinfo)
  */
 static inline void xbus_drift_clear(xbus_t *xbus)
 {
-	struct xpp_drift	*driftinfo = &xbus->drift;
+	struct xpp_drift	*di = &xbus->drift;
 
-	driftinfo_recalc(driftinfo);
-	driftinfo->calc_drift = 0;
+	do_gettimeofday(&di->last_lost_tick.tv);
 	ticker_set_cycle(&xbus->ticker, SYNC_ADJ_QUICK);
+	di->max_speed = -SYNC_ADJ_MAX;
+	di->min_speed =  SYNC_ADJ_MAX;
 }
 
 void xpp_drift_init(xbus_t *xbus)
@@ -148,7 +150,6 @@ void xpp_drift_init(xbus_t *xbus)
 	memset(&xbus->drift, 0, sizeof(xbus->drift));
 	spin_lock_init(&xbus->drift.lock);
 	xpp_ticker_init(&xbus->ticker);
-	xbus->drift.wanted_offset = 500;
 	xbus_drift_clear(xbus);
 }
 
@@ -168,110 +169,128 @@ static void sample_tick(xbus_t *xbus, int sample)
 #define	sample_tick(x,y)
 #endif
 
+/*
+ * The following function adjust the clock of an astribank according
+ * to our reference clock (another astribank or another DAHDI device).
+ *
+ * It is VERY hard to stabilise these corrections:
+ *    - The measurments are affected by 125usec USB micro-frames.
+ *    - However, if we are off by more than +/-500usec we are risking
+ *      loosing frames.
+ *
+ * Every change must be tested rigorously with many device combinations
+ * including abrupt changes in sync -- to verify the stability of the
+ * algorithm.
+ */
 static void xpp_drift_step(xbus_t *xbus, const struct timeval *tv)
 {
-	struct xpp_drift	*driftinfo = &xbus->drift;
+	struct xpp_drift	*di = &xbus->drift;
 	struct xpp_ticker	*ticker = &xbus->ticker;
 	unsigned long		flags;
-	bool			cycled;
 
-	spin_lock_irqsave(&driftinfo->lock, flags);
-	cycled = xpp_ticker_step(&xbus->ticker, tv);
+	spin_lock_irqsave(&di->lock, flags);
+	xpp_ticker_step(&xbus->ticker, tv);
+	/*
+	 * Do we need to be synchronized and is there an established reference
+	 * ticker (another Astribank or another DAHDI device) already?
+	 */
 	if(ref_ticker && ref_ticker != &xbus->ticker && syncer && xbus->sync_mode == SYNC_MODE_PLL) {
 		int	new_delta_tick = ticker->count - ref_ticker->count;
-		int	lost_ticks = new_delta_tick - driftinfo->delta_tick;
+		int	lost_ticks = new_delta_tick - di->delta_tick;
+		long	usec_delta;
 
-		driftinfo->delta_tick = new_delta_tick;
+		di->delta_tick = new_delta_tick;
 		if(lost_ticks) {
 			static int	rate_limit;
 
-			driftinfo->lost_ticks++;
-			driftinfo->lost_tick_count += abs(lost_ticks);
-
+			/*
+			 * We just lost some ticks. Just report it and don't
+			 * try to adjust anything until we are stable again.
+			 */
+			di->lost_ticks++;
+			di->lost_tick_count += abs(lost_ticks);
 			if((rate_limit++ % 1003) == 0) {
+				/* FIXME: This should be a NOTICE.
+				 * However we have several false ones at
+				 * startup.
+				 */
 				XBUS_DBG(SYNC, xbus, "Lost %d tick%s\n",
 					lost_ticks,
 					(abs(lost_ticks) > 1) ? "s": "");
 			}
-			ticker_set_cycle(ticker, SYNC_ADJ_QUICK);
-			if(abs(lost_ticks) > 100)
+			if(abs(lost_ticks) > 100) {
+				xbus_drift_clear(xbus);
 				ticker->count = ref_ticker->count;
+			}
 		} else {
-			long	usec_delta;
-			bool	nofix = 0;
-
+			/* Sample a delta */
 			usec_delta = (long)usec_diff(
 					&ticker->last_sample.tv,
 					&ref_ticker->last_sample.tv);
-			usec_delta -= driftinfo->wanted_offset;
 			sample_tick(xbus, usec_delta);
-			if(abs(usec_delta) > 300) {
+			if ((ticker->count % SYNC_CYCLE) > (SYNC_CYCLE - SYNC_CYCLE_SAMPLE))
+				di->delta_sum  += usec_delta;
+					
+			if((ticker->count % SYNC_CYCLE) == 0) {
 				/*
-				 * We are close to the edge, send a brutal
-				 * fix, and skip calculation until next time.
+				 * Full sampling cycle passed. Let's calculate
 				 */
-				if(usec_delta > 0 && xbus->sync_adjustment > -SYNC_ADJ_MAX) {
-					XBUS_DBG(SYNC, xbus, "Pullback usec_delta=%ld\n", usec_delta);
-					driftinfo->kicks_down++;
-					send_drift(xbus, -SYNC_ADJ_MAX);	/* emergency push */
-				}
-				if(usec_delta < 0 && xbus->sync_adjustment < SYNC_ADJ_MAX) {
-					XBUS_DBG(SYNC, xbus, "Pushback usec_delta=%ld\n", usec_delta);
-					driftinfo->kicks_up++;
-					send_drift(xbus, SYNC_ADJ_MAX);		/* emergency push */
-				}
-				ticker_set_cycle(ticker, SYNC_ADJ_QUICK);
-				nofix = 1;
-			} else {
-				/* good data, use it */
-				if(usec_delta > driftinfo->delta_max)
-					driftinfo->delta_max = usec_delta;
-				if(usec_delta < driftinfo->delta_min)
-					driftinfo->delta_min = usec_delta;
-			}
-			if(!nofix && cycled) {
-				int	offset = 0;
+				int	offset		= di->delta_sum / SYNC_CYCLE_SAMPLE - SYNC_CENTER;
+				int	offset_prev	= di->offset_prev;
+				int	speed		= xbus->sync_adjustment;
+				int	fix		= 0;
+				int	best_speed	= (di->max_speed + di->min_speed) >> 1;
 
-				driftinfo->median = (driftinfo->delta_max + driftinfo->delta_min) / 2;
-				driftinfo->jitter = driftinfo->delta_max - driftinfo->delta_min;
-				if(abs(driftinfo->median) >= 150) {	/* more than 1 usb uframe */
-					int	factor = abs(driftinfo->median) / 125;
-
-					factor = 1 + (factor * 8000) / ticker->cycle;
-					if(driftinfo->median > 0)
-						offset = driftinfo->calc_drift - factor;
-					else
-						offset = driftinfo->calc_drift + factor;
-					/* for large median, push some more */
-					if(abs(driftinfo->median) >= 300) {	/* more than 2 usb uframes */
-						ticker_set_cycle(ticker, SYNC_ADJ_QUICK);
-						XBUS_NOTICE(xbus,
-								"Back to quick: median=%d\n",
-								driftinfo->median);
-					}
+				if      (offset > 0 && offset < SYNC_DELTA) {
+					speed = best_speed - 1;
+				} else if (offset < 0 && offset > -SYNC_DELTA) {
+					speed = best_speed + 1;
 				} else {
-					//ticker_set_cycle(ticker, ticker->cycle + 500);
+					if (offset > 0) {
+						if (offset > offset_prev) 
+							fix--;
+					} else {
+						if (offset < offset_prev) 
+							fix++;
+					}
+					speed += fix;
 				}
-				driftinfo->calc_drift = offset;
-				XBUS_DBG(SYNC, xbus,
-						"ADJ: min=%d max=%d jitter=%d median=%d offset=%d\n",
-						driftinfo->delta_min,
-						driftinfo->delta_max,
-						driftinfo->jitter,
-						driftinfo->median,
-						offset);
-				if(offset < -SYNC_ADJ_MAX)
-					offset = -SYNC_ADJ_MAX;
-				if(offset > SYNC_ADJ_MAX)
-					offset = SYNC_ADJ_MAX;
-				xbus->sync_adjustment_offset = offset;
-				if(xbus != syncer && xbus->sync_adjustment != offset)
-					send_drift(xbus, offset);
-				driftinfo_recalc(driftinfo);
+				if (speed < -SYNC_ADJ_MAX)
+					speed = -SYNC_ADJ_MAX;
+				if (speed > SYNC_ADJ_MAX)
+					speed =  SYNC_ADJ_MAX;
+				if (speed < di->min_speed)
+					di->min_speed    = speed;
+				if (speed > di->max_speed)
+					di->max_speed    = speed;
+				if(offset > di->offset_max)
+					di->offset_max = offset;
+				if(offset < di->offset_min)
+					di->offset_min = offset;
+
+				XBUS_DBG(SYNC, xbus, "offset: %d, min_speed=%d, max_speed=%d, usec_delta(last)=%ld\n",
+					 offset_prev, di->min_speed, di->max_speed, usec_delta);
+				XBUS_DBG(SYNC, xbus, "ADJ: speed=%d (best_speed=%d) fix=%d\n", 
+					speed, best_speed, fix);
+				xbus->sync_adjustment_offset = speed;
+				if(xbus != syncer && xbus->sync_adjustment != speed)
+					send_drift(xbus, speed);
+				di->sync_inaccuracy = abs(offset) + abs(di->offset_range) / 2;
+				if(ticker->count >= SYNC_CYCLE * SYNC_CONVERGE) {
+					di->offset_range = di->offset_max - di->offset_min;
+					di->offset_min = INT_MAX;
+					di->offset_max = -INT_MAX;
+					if(di->max_speed > best_speed)
+						di->max_speed--;
+					if(di->min_speed < best_speed)
+						di->min_speed++;
+				}
+				di->offset_prev = offset;
+				di->delta_sum      = 0;
 			}
 		}
 	}
-	spin_unlock_irqrestore(&driftinfo->lock, flags);
+	spin_unlock_irqrestore(&di->lock, flags);
 }
 
 const char *sync_mode_name(enum sync_mode mode)
@@ -287,17 +306,17 @@ const char *sync_mode_name(enum sync_mode mode)
 	return sync_mode_names[mode];
 }
 
+/*
+ * Caller must aquire/release the 'ref_ticker_lock' spinlock
+ */
 static void xpp_set_syncer(xbus_t *xbus, bool on)
 {
-	unsigned long	flags;
-
-	spin_lock_irqsave(&ref_ticker_lock, flags);
 	if(!xbus) {	/* Special case, no more syncers */
 		DBG(SYNC, "No more syncers\n");
 		syncer = NULL;
 		if(ref_ticker != &dahdi_ticker)
 			ref_ticker = NULL;
-		goto out;
+		return;
 	}
 	if(syncer != xbus && on) {
 		XBUS_DBG(SYNC, xbus, "New syncer\n");
@@ -311,8 +330,6 @@ static void xpp_set_syncer(xbus_t *xbus, bool on)
 		XBUS_DBG(SYNC, xbus, "ignore %s (current syncer: %s)\n",
 			(on)?"ON":"OFF",
 			(syncer) ? syncer->busname : "NO-SYNC");
-out:
-	spin_unlock_irqrestore(&ref_ticker_lock, flags);
 }
 
 static void xbus_command_timer(unsigned long param)
@@ -351,11 +368,14 @@ void xbus_set_command_timer(xbus_t *xbus, bool on)
 void got_new_syncer(xbus_t *xbus, enum sync_mode mode, int drift)
 {
 	unsigned long	flags;
+	unsigned long	flags2;
 
 	spin_lock_irqsave(&xbus->lock, flags);
+	spin_lock_irqsave(&ref_ticker_lock, flags2);
 	xbus->sync_adjustment = (signed char)drift;
 	if(xbus->sync_mode == mode) {
-		/* XBUS_DBG(SYNC, xbus, "Already in mode '%s'. Ignored\n", sync_mode_name(mode)); */
+		XBUS_DBG(SYNC, xbus, "Already in mode '%s'. Ignored\n",
+				sync_mode_name(mode));
 		goto out;
 	}
 	XBUS_DBG(SYNC, xbus, "Mode %s (%d), drift=%d (pcm_rx_counter=%d)\n",
@@ -384,15 +404,32 @@ void got_new_syncer(xbus_t *xbus, enum sync_mode mode, int drift)
 		XBUS_ERR(xbus, "%s: unknown mode=0x%X\n", __FUNCTION__, mode);
 	}
 out:
+	spin_unlock_irqrestore(&ref_ticker_lock, flags2);
 	spin_unlock_irqrestore(&xbus->lock, flags);
 }
 
 void xbus_request_sync(xbus_t *xbus, enum sync_mode mode)
 {
+	unsigned long	flags;
+
 	BUG_ON(!xbus);
 	XBUS_DBG(SYNC, xbus, "sent request (mode=%d)\n", mode);
 	CALL_PROTO(GLOBAL, SYNC_SOURCE, xbus, NULL, mode, 0);
 	if(mode == SYNC_MODE_NONE) {
+		/*
+		 * We must deselect the syncer *now* and not wait for the
+		 * reply from the AB. Otherwise, a disconnect of the syncing
+		 * AB would result in a corrupted 'syncer'.
+		 */
+		spin_lock_irqsave(&ref_ticker_lock, flags);
+		xpp_set_syncer(xbus, 0);
+		spin_unlock_irqrestore(&ref_ticker_lock, flags);
+		/*
+		 * We must activate timer now since the commands go to
+		 * the command queue, and we should maintain something to
+		 * "tick" meanwhile until the AB get these commands and
+		 * start being "self_ticking".
+		 */
 		xbus_set_command_timer(xbus, 1);
 	}
 }
@@ -403,9 +440,9 @@ static void reset_sync_counters(void)
 
 	//DBG(SYNC, "%d\n", atomic_read(&xpp_tick_counter));
 	for(i = 0; i < MAX_BUSES; i++) {
-		xbus_t	*xbus = xbus_num(i);
+		xbus_t	*xbus = get_xbus(__func__, i);
 
-		if(!xbus)
+		if (xbus == NULL)
 			continue;
 		/*
 		 * Don't send to non self_ticking Astribanks:
@@ -422,6 +459,7 @@ static void reset_sync_counters(void)
 				CALL_PROTO(GLOBAL, RESET_SYNC_COUNTERS, xbus, NULL);
 			}
 		}
+		put_xbus(__func__, xbus);
 	}
 }
 
@@ -458,7 +496,7 @@ static void global_tick(void)
 #ifdef	DAHDI_SYNC_TICK
 int dahdi_sync_tick(struct dahdi_span *span, int is_master)
 {
-	xpd_t		*xpd = span->pvt;
+	xpd_t		*xpd = container_of(span, struct xpd, span);
 	static int	redundant_ticks;	/* for extra spans */
 	struct timeval	now;
 
@@ -541,6 +579,7 @@ static void update_sync_master(xbus_t *new_syncer, bool force_dahdi)
 		force_dahdi_sync = 0;
 		ref_ticker = &new_syncer->ticker;
 		xbus_drift_clear(new_syncer);	/* Clean new data */
+		xpp_set_syncer(new_syncer, 1);
 		xbus_request_sync(new_syncer, SYNC_MODE_AB);
 	} else if(force_dahdi_sync) {
 		ref_ticker = &dahdi_ticker;
@@ -551,15 +590,17 @@ static void update_sync_master(xbus_t *new_syncer, bool force_dahdi)
 	DBG(SYNC, "stop unwanted syncers\n");
 	/* Shut all down except the wanted sync master */
 	for(i = 0; i < MAX_BUSES; i++) {
-		xbus_t	*xbus = xbus_num(i);
-		if(!xbus)
+		xbus_t	*xbus = get_xbus(__func__, i);
+		if (xbus == NULL)
 			continue;
 		if(XBUS_FLAGS(xbus, CONNECTED) && xbus != new_syncer) {
 			if(xbus->self_ticking)
-				xbus_request_sync(xbus, SYNC_MODE_PLL);
+				xbus_request_sync(xbus,
+						xbus->sync_mode_default);
 			else
 				XBUS_DBG(SYNC, xbus, "Not self_ticking yet. Ignore\n");
 		}
+		put_xbus(__func__, xbus);
 	}
 }
 
@@ -570,25 +611,39 @@ void elect_syncer(const char *msg)
 	uint	timing_priority = INT_MAX;
 	xpd_t	*best_xpd = NULL;
 	xbus_t	*the_xbus = NULL;
+	unsigned long	flags;
 
+	spin_lock_irqsave(&elect_syncer_lock, flags);
+	DBG(SYNC, "%s: %s syncer=%s\n", __func__, msg,
+			(syncer) ? syncer->busname : "NULL");
 	for(i = 0; i < MAX_BUSES; i++) {
-		xbus_t	*xbus = xbus_num(i);
-		if(!xbus)
+		xbus_t	*xbus = get_xbus(__func__, i);
+
+		if (xbus == NULL)
 			continue;
 		if(XBUS_IS(xbus, READY)) {
 			if(!the_xbus)
 				the_xbus = xbus;	/* First candidate */
 			for(j = 0; j < MAX_XPDS; j++) {
 				xpd_t	*xpd = xpd_of(xbus, j);
+				int	prio;
 
 				if(!xpd || !xpd->card_present)
 					continue;
-				if(xpd->timing_priority > 0 && xpd->timing_priority < timing_priority) {
-					timing_priority = xpd->timing_priority;
+				prio = CALL_XMETHOD(card_timing_priority,
+						xbus, xpd);
+				if (prio < 0) {
+					DBG(SYNC, "%s/%s: skip sync\n",
+						xbus->busname, xpd->xpdname);
+					continue;
+				}
+				if (prio > 0 && prio < timing_priority) {
+					timing_priority = prio;
 					best_xpd = xpd;
 				}
 			}
 		}
+		put_xbus(__func__, xbus);
 	}
 	if(best_xpd) {
 		the_xbus = best_xpd->xbus;
@@ -596,12 +651,28 @@ void elect_syncer(const char *msg)
 	} else if(the_xbus) {
 		XBUS_DBG(SYNC, the_xbus, "%s: elected\n", msg);
 	} else {
+		unsigned long	flags;
+
 		DBG(SYNC, "%s: No more syncers\n", msg);
+		spin_lock_irqsave(&ref_ticker_lock, flags);
 		xpp_set_syncer(NULL, 0);
+		spin_unlock_irqrestore(&ref_ticker_lock, flags);
 		the_xbus = NULL;
 	}
 	if(the_xbus != syncer)
 		update_sync_master(the_xbus, force_dahdi_sync);
+	spin_unlock_irqrestore(&elect_syncer_lock, flags);
+}
+
+/*
+ * This function should be called with the xpd already locked
+ */
+void update_wanted_pcm_mask(xpd_t *xpd, xpp_line_t new_mask, uint new_pcm_len)
+{
+	xpd->pcm_len = new_pcm_len;
+	xpd->wanted_pcm_mask = new_mask;
+	XPD_DBG(SIGNAL, xpd, "pcm_len=%d wanted_pcm_mask=0x%X\n",
+		xpd->pcm_len, xpd->wanted_pcm_mask);
 }
 
 /*
@@ -614,6 +685,7 @@ void generic_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd, xpp_line_t pcm_mask)
 	int		i;
 	int		line_count = 0;
 	unsigned long	flags;
+	uint		pcm_len;
 
 	spin_lock_irqsave(&xpd->lock_recompute_pcm, flags);
 	//XPD_DBG(SIGNAL, xpd, "pcm_mask=0x%X\n", pcm_mask);
@@ -633,12 +705,10 @@ void generic_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd, xpp_line_t pcm_mask)
 		pcm_mask = BIT(0);
 		line_count = 1;
 	}
-	xpd->pcm_len = (line_count)
+	pcm_len = (line_count)
 		? RPACKET_HEADERSIZE + sizeof(xpp_line_t) + line_count * DAHDI_CHUNKSIZE
 		: 0L;
-	xpd->wanted_pcm_mask = pcm_mask;
-	XPD_DBG(SIGNAL, xpd, "pcm_len=%d wanted_pcm_mask=0x%X\n",
-		xpd->pcm_len, xpd->wanted_pcm_mask);
+	update_wanted_pcm_mask(xpd, pcm_mask, pcm_len);
 	spin_unlock_irqrestore(&xpd->lock_recompute_pcm, flags);
 }
 
@@ -934,6 +1004,11 @@ out:
 	return ret;
 }
 
+int generic_timing_priority(xbus_t *xbus, xpd_t *xpd)
+{
+	return xpd->timing_priority;
+}
+
 static void xbus_tick(xbus_t *xbus)
 {
 	int		i;
@@ -1108,18 +1183,22 @@ int exec_sync_command(const char *buf, size_t count)
 		update_sync_master(NULL, 1);
 	} else if(sscanf(buf, "SYNC=%d", &xbusno) == 1) {
 		DBG(SYNC, "SYNC=%d\n", xbusno);
-		if((xbus = xbus_num(xbusno)) == NULL) {
+		xbus = get_xbus(__func__, xbusno);
+		if (xbus == NULL) {
 			ERR("No bus %d exists\n", xbusno);
 			return -ENXIO;
 		}
 		update_sync_master(xbus, 0);
+		put_xbus(__func__, xbus);
 	} else if(sscanf(buf, "QUERY=%d", &xbusno) == 1) {
 		DBG(SYNC, "QUERY=%d\n", xbusno);
-		if((xbus = xbus_num(xbusno)) == NULL) {
+		xbus = get_xbus(__func__, xbusno);
+		if (xbus == NULL) {
 			ERR("No bus %d exists\n", xbusno);
 			return -ENXIO;
 		}
 		CALL_PROTO(GLOBAL, SYNC_SOURCE, xbus, NULL, SYNC_MODE_QUERY, 0);
+		put_xbus(__func__, xbus);
 	} else {
 		ERR("%s: cannot parse '%s'\n", __FUNCTION__, buf);
 		ret = -EINVAL;
@@ -1206,7 +1285,7 @@ static struct proc_dir_entry	*top;
 #endif
 #endif	/* OLD_PROC */
 
-int xbus_pcm_init(struct proc_dir_entry *toplevel)
+int xbus_pcm_init(void *toplevel)
 {
 	int			ret = 0;
 
@@ -1258,9 +1337,11 @@ EXPORT_SYMBOL(elect_syncer);
 #ifdef	DAHDI_SYNC_TICK
 EXPORT_SYMBOL(dahdi_sync_tick);
 #endif
+EXPORT_SYMBOL(update_wanted_pcm_mask);
 EXPORT_SYMBOL(generic_card_pcm_recompute);
 EXPORT_SYMBOL(generic_card_pcm_tospan);
 EXPORT_SYMBOL(generic_card_pcm_fromspan);
+EXPORT_SYMBOL(generic_timing_priority);
 #ifdef	DEBUG_PCMTX
 EXPORT_SYMBOL(pcmtx);
 EXPORT_SYMBOL(pcmtx_chan);
