@@ -314,13 +314,17 @@ enum {
 #define DAHDI_CONF_PSEUDO_TALKER	0x800		/* pseudo is a talker on the conference */
 
 /* Alarm Condition bits */
-#define DAHDI_ALARM_NONE		0	/* No alarms */
-#define DAHDI_ALARM_RECOVER		1	/* Recovering from alarm */
-#define DAHDI_ALARM_LOOPBACK		2	/* In loopback */
-#define DAHDI_ALARM_YELLOW		4	/* Yellow Alarm */
-#define DAHDI_ALARM_RED			8	/* Red Alarm */
-#define DAHDI_ALARM_BLUE		16	/* Blue Alarm */
-#define DAHDI_ALARM_NOTOPEN		32
+#define DAHDI_ALARM_NONE		0	 /* No alarms */
+#define DAHDI_ALARM_RECOVER		(1 << 0) /* Recovering from alarm */
+#define DAHDI_ALARM_LOOPBACK		(1 << 1) /* In loopback */
+#define DAHDI_ALARM_YELLOW		(1 << 2) /* Yellow Alarm */
+#define DAHDI_ALARM_RED			(1 << 3) /* Red Alarm */
+#define DAHDI_ALARM_BLUE		(1 << 4) /* Blue Alarm */
+#define DAHDI_ALARM_NOTOPEN		(1 << 5)
+/* Verbose alarm states (upper byte) */
+#define DAHDI_ALARM_LOS			(1 << 8) /* Loss of Signal */
+#define DAHDI_ALARM_LFA			(1 << 9) /* Loss of Frame Alignment */
+#define DAHDI_ALARM_LMFA		(1 << 10)/* Loss of Multi-Frame Align */
 
 /* Maintenance modes */
 #define DAHDI_MAINT_NONE		0	/* Normal Mode */
@@ -338,7 +342,8 @@ enum {
 #define DAHDI_MAINT_BIPOLAR_DEFECT	11	/* insert a FAS defect */
 #define DAHDI_MAINT_PRBS		12	/* enable the PRBS gen/mon */
 #define DAHDI_MAINT_NETWORKPAYLOADLOOP	13	/* Remote Loopback */
-#define DAHDI_RESET_COUNTERS		14	/* clear the error counters */
+#define DAHDI_RESET_COUNTERS		14	/* Clear the error counters */
+#define DAHDI_MAINT_ALARM_SIM		15	/* Simulate alarms */
 
 /* Flag Value for IOMUX, read avail */
 #define DAHDI_IOMUX_READ	1
@@ -566,7 +571,8 @@ struct dahdi_spaninfo {
 	char	manufacturer[40]; /* manufacturer of span's device */
 	char	devicetype[40];	/* span's device type */
 	int	irq;		/* span's device IRQ */
-	int	linecompat;	/* signaling modes possible on this span */
+	int	linecompat;	/* span global signaling or 0 for
+				   analog spans.*/
 	char	spantype[6];	/* type of span in text form */
 } __attribute__((packed));
 
